@@ -18,6 +18,10 @@
 #	include <hildon/hildon-fm.h>
 #endif
 
+#ifdef HAVE_UNITY
+	#include <unity/unity/unity.h>
+#endif
+
 #include "callbacks.h"
 #include "support.h"
 
@@ -39,6 +43,7 @@ FILE *log_file = NULL;
 glong scroll_previous_run;
 GtkStatusIcon *trayIcon = NULL;
 GtkTreePath *path_set = NULL;
+gboolean more = FALSE, first = TRUE;
 
 
 void on_trayicon_activate(GObject *trayIcon, gpointer window) {
@@ -110,6 +115,13 @@ void set_main_title(gchar* session, gchar* extra) {
 }
 
 
+void set_rsync_title(gchar* session, gchar* extra) {
+	gchar *stmp = g_strconcat("rsync: ", session, " ", extra, NULL);
+	gtk_window_set_title((GtkWindow*) rsync_window, stmp);
+	g_free(stmp);
+}
+
+
 void load_settings(gchar* session, gchar *filename) {
 	gchar settings_file_path[MAXPATH], *stmp;
 	static gboolean resized = FALSE;
@@ -188,7 +200,8 @@ void load_settings(gchar* session, gchar *filename) {
 			enabled = g_key_file_get_boolean(settings_file, session, stmp, NULL);
 			g_free(stmp);
 
-			if (enabled) gtk_list_store_set(liststore_session, &iter, 1, TRUE, -1);
+			// if (enabled) gtk_list_store_set(liststore_session, &iter, 1, TRUE, -1);
+			gtk_list_store_set(liststore_session, &iter, 1, enabled, -1);
 			gtk_tree_path_next(path);
 			i++;
 		}
@@ -393,7 +406,7 @@ gboolean load_groups(gchar *session) {
 		GKeyFile *settings_file;
 		GtkComboBox *combo;
 		GtkTreeIter iter;
-		GtkTreeModelFilter *filter = (GtkTreeModelFilter*) gtk_builder_get_object(builder, "treemodelfilter_set");
+		static GtkTreeModelFilter *filter = NULL;
 		gint i;
 		gboolean flag = FALSE;
 
@@ -441,8 +454,12 @@ gboolean load_groups(gchar *session) {
 			}
 			i++;
 		}
-		gtk_tree_model_filter_set_visible_column(filter, 2);
-		gtk_tree_model_filter_refilter(filter);
+
+		if (!filter) {
+			filter = (GtkTreeModelFilter*) gtk_builder_get_object(builder, "treemodelfilter_set");
+			gtk_tree_model_filter_set_visible_column(filter, 2);
+			gtk_tree_model_filter_refilter(filter);
+		}
 
 		g_key_file_free(settings_file);
 		g_clear_error(&error_handler);
@@ -514,11 +531,21 @@ void show_browse_dest(GtkButton *button, gpointer user_data) {
 void set_global_progress(gdouble fraction) {
 	GTimeVal curtime;
 	glong elapsed, remaining;
+	static glong lastupdate = 0;
 	gchar tmps[50];
 	GtkProgressBar* progr = (GtkProgressBar*) gtk_builder_get_object(builder, "progress_global");
 
-	gtk_progress_bar_set_fraction(progr, fraction);
 	g_get_current_time(&curtime);
+	if (fraction > (gdouble)0 && fraction < (gdouble)1 && curtime.tv_sec == lastupdate) return;
+	lastupdate = curtime.tv_sec;
+
+	#ifdef HAVE_UNITY
+		UnityLauncherEntry *launcher_entry = unity_launcher_entry_get_for_desktop_id("grsync.desktop");
+		unity_launcher_entry_set_progress(launcher_entry, fraction);
+		unity_launcher_entry_set_progress_visible(launcher_entry, fraction > (gdouble)0);
+ 	#endif
+
+	gtk_progress_bar_set_fraction(progr, fraction);
 	elapsed = curtime.tv_sec - startime.tv_sec;
 	remaining = elapsed / fraction - elapsed;
 	if (fraction) snprintf(tmps, 49, _("%.f%% (%li:%02li elapsed, %li:%02li remaining)"), round(fraction * 100), elapsed / 60, elapsed % 60, remaining / 60, remaining % 60);
@@ -529,9 +556,15 @@ void set_global_progress(gdouble fraction) {
 
 
 void set_file_progress(gdouble fraction) {
+	GTimeVal curtime;
+	static glong lastupdate = 0;
 	gchar tmps[50];
 	GtkProgressBar* progr = (GtkProgressBar*) gtk_builder_get_object(builder, "progress_file");
 
+	g_get_current_time(&curtime);
+	if (fraction < (gdouble)1 && curtime.tv_sec == lastupdate) return;
+	lastupdate = curtime.tv_sec;
+
 	gtk_progress_bar_set_fraction(progr, fraction);
 	snprintf(tmps, 5, "%.f%%", round(fraction * 100));
 	gtk_progress_bar_set_text(progr, tmps);
@@ -605,6 +638,9 @@ gboolean set_next() {
 		session_set = session_last;
 		path_set = gtk_tree_path_new_first();
 		i = 0;
+		first = TRUE;
+	} else {
+		first = FALSE;
 	}
 
 	while ((more = gtk_tree_model_get_iter((GtkTreeModel*) liststore_session, &iter, path_set))) {
@@ -629,12 +665,13 @@ gboolean set_next() {
 }
 
 
-gboolean on_play_clicked(GtkButton *button, gpointer user_data) {
+void on_play_clicked(GtkButton *button, gpointer user_data) {
 	gchar *gtmp;
-	gboolean more = FALSE;
-
-	if (is_set) more = set_next();
 
+	if (is_set) {
+		more = set_next();
+		if (!more) return;
+	}
 	build_rsync_cmd();
 	paused = FALSE;
 	// scroll_previous_run = 0;
@@ -643,15 +680,18 @@ gboolean on_play_clicked(GtkButton *button, gpointer user_data) {
 	// with expander initialized, let's resize the window:
 	on_expander_rsync_activate((GtkExpander*) gtk_builder_get_object(builder, "expander_rsync"), NULL);
 
-	if (error_list != NULL) g_free(error_list);
-	error_list = g_strconcat("", NULL);
+	if (!is_set || first) {
+		if (error_list != NULL) g_free(error_list);
+		error_list = g_strconcat("", NULL);
+	}
+
 	if (config_log) {
 		gtmp = g_strconcat(grsync_dir, "/", groups[session_last], ".log", NULL);
 		log_file = fopen(gtmp, "w");
 		g_free(gtmp);
 	}
 
-	gtk_window_set_title((GtkWindow*) rsync_window, _("rsync: running"));
+	set_rsync_title(groups[session_last], "running");
 	gtk_button_set_label((GtkButton*) gtk_builder_get_object(builder, "close"), "gtk-stop");
 	gtk_button_set_label((GtkButton*) gtk_builder_get_object(builder, "button_pause"), "gtk-media-pause");
 	gtk_widget_set_sensitive((GtkWidget*) gtk_builder_get_object(builder, "button_pause"), TRUE);
@@ -661,8 +701,8 @@ gboolean on_play_clicked(GtkButton *button, gpointer user_data) {
 	set_file_progress(0);
 	set_global_progress(0);
 	gtk_label_set_text((GtkLabel*) gtk_builder_get_object(builder, "label_file"), dryrunning ? _("Simulation in progress") : "");
-	gtk_widget_show(rsync_window);
-	return more;
+	if (!is_set || first) gtk_widget_show(rsync_window);
+	else on_rsync_show(NULL, NULL);
 }
 
 
@@ -796,8 +836,11 @@ void on_close_clicked(GtkButton *button, gpointer user_data) {
 		kill(rsync_pid, SIGTERM);
 		gtk_widget_set_sensitive((GtkWidget*) gtk_builder_get_object(builder, "button_pause"), FALSE);
 	} else {
-		if (on_play_clicked(NULL, NULL)) on_rsync_show(NULL, NULL);
-		else gtk_widget_hide(rsync_window);
+		if (more) on_play_clicked(NULL, NULL);		//on_rsync_show(NULL, NULL);
+		else {
+			set_global_progress(0);
+			gtk_widget_hide(rsync_window);
+		}
 	}
 	if (cmdline_execute) on_main_destroy((GtkObject*) main_window, NULL);
 }
@@ -835,7 +878,7 @@ void rsync_cleanup(gpointer data) {
 				comm_out = g_strconcat("", NULL);
 				comm_err = g_strconcat("Error launching command!\n", NULL);
 			}
-			comm_after = g_strconcat("\n", _("*** Launching AFTER command:\n"), comm_str, "\n", comm_out, comm_err, NULL);
+			comm_after = g_strconcat("\n", _("** Launching AFTER command:\n"), comm_str, "\n", comm_out, comm_err, NULL);
 			g_free(comm_out);
 			g_free(comm_err);
 
@@ -858,7 +901,7 @@ void rsync_cleanup(gpointer data) {
 	set_global_progress(1);
 	gtk_widget_set_sensitive((GtkWidget*) gtk_builder_get_object(builder, "button_pause"), FALSE);
 	gtk_widget_set_sensitive((GtkWidget*) view, TRUE);
-	gtk_window_set_title((GtkWindow*) rsync_window, _("rsync: stopped"));
+	set_rsync_title(groups[session_last], "stopped");
 
 	g_io_channel_unref(chout);
 	g_io_channel_unref(cherr);
@@ -872,7 +915,7 @@ void rsync_cleanup(gpointer data) {
 	if (config_errorlist && had_error) on_button_show_errors_clicked(NULL, NULL);
 	else gtk_window_set_urgency_hint((GtkWindow*) rsync_window, TRUE);
 	set_trayicon_icon(FALSE);
-	if (cmdline_execute && !(config_errorlist && had_error) && !cmdline_stayopen) on_close_clicked(NULL, NULL);
+	if (more || (cmdline_execute && !(config_errorlist && had_error) && !cmdline_stayopen)) on_close_clicked(NULL, NULL);
 
 	if (log_file != NULL) {
 		fclose(log_file);
@@ -1035,7 +1078,7 @@ void on_rsync_show(GtkWidget *widget, gpointer user_data) {
 	gboolean halt = FALSE;
 
 	rsync_pid = 0;
-	had_error = FALSE;
+	if (!is_set || first) had_error = FALSE;
 	gtk_label_set_ellipsize((GtkLabel*) gtk_builder_get_object(builder, "label_file"), PANGO_ELLIPSIZE_MIDDLE);
 
 	if (get_checkbox("check_com_before")) {
@@ -1045,7 +1088,7 @@ void on_rsync_show(GtkWidget *widget, gpointer user_data) {
 			comm_err = g_strconcat("Error launching command!\n", NULL);
 			if (get_checkbox("check_com_halt")) halt = TRUE;
 		}
-		comm_before = g_strconcat(_("*** Launching BEFORE command:\n"), comm_str, "\n", comm_out, comm_err, "\n", NULL);
+		comm_before = g_strconcat(_("** Launching BEFORE command:\n"), comm_str, "\n", comm_out, comm_err, "\n", NULL);
 		g_free(comm_out);
 		g_free(comm_err);
 	} else {
@@ -1055,16 +1098,17 @@ void on_rsync_show(GtkWidget *widget, gpointer user_data) {
 	comm_str = g_strjoinv(" ", argv);
 	if (halt) tmpc = "";
 	else {
-		if (dryrunning) tmpc =_("*** Launching RSYNC command (simulation mode):\n");
-		else tmpc = _("*** Launching RSYNC command:\n");
+		if (dryrunning) tmpc =_("** Launching RSYNC command (simulation mode):\n");
+		else tmpc = _("** Launching RSYNC command:\n");
 	}
-	comm_text = g_strconcat(comm_before, tmpc, comm_str, "\n\n", NULL);
+	comm_text = g_strconcat("**** ", groups[session_last], "\n\n", comm_before, tmpc, comm_str, "\n\n", NULL);
 	g_free(comm_before);
 	g_free(comm_str);
 
 	view = (GtkTextView*)gtk_builder_get_object(builder, "textview_output");
 	buffer = gtk_text_view_get_buffer(view);
-	gtk_text_buffer_set_text(buffer, "", -1);
+	if (is_set && !first) gtk_text_buffer_insert_at_cursor(buffer, "\n", -1);
+	else gtk_text_buffer_set_text(buffer, "", -1);
 	gtk_text_buffer_insert_at_cursor(buffer, comm_text, -1);
 	if (config_log) fputs(comm_text, log_file);
 	g_free(comm_text);
@@ -1170,10 +1214,12 @@ void on_session_add_clicked(GtkButton *button, gpointer user_data) {
 			return;
 		}
 
+		save_settings(groups[session_last], NULL);
 		save_settings(newses, NULL);
 		session_number++;
 		session_last = -1;		// added because of save session bug, copied from external patch
 		load_groups(newses);
+		load_settings(newses, NULL);
 	}
 	gtk_widget_hide(add);
 }
@@ -1288,14 +1334,14 @@ void on_button_pause_clicked(GtkButton *button, gpointer user_data) {
 	if (paused) {
 		if (rsync_pid) kill(rsync_pid, SIGCONT);
 		gtk_button_set_label(tmpbutt, "gtk-media-pause");
-		gtk_window_set_title((GtkWindow*) rsync_window, _("rsync: running"));
+		set_rsync_title(groups[session_last], "running");
 		g_get_current_time(&tmptime);
 		startime.tv_sec += tmptime.tv_sec - pausedtime.tv_sec;
 		startime.tv_usec += tmptime.tv_usec - pausedtime.tv_usec;
 	} else {
 		if (rsync_pid) kill(rsync_pid, SIGSTOP);
 		gtk_button_set_label(tmpbutt, "gtk-media-play");
-		gtk_window_set_title((GtkWindow*) rsync_window, _("rsync: paused"));
+		set_rsync_title(groups[session_last], "paused");
 		g_get_current_time(&pausedtime);
 		g_idle_add(scroll_idle, gtk_builder_get_object(builder, "textview_output"));
 	}
@@ -1410,7 +1456,7 @@ void on_import1_activate(GtkMenuItem *menuitem, gpointer user_data) {
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 #else
 	dialog = hildon_file_chooser_dialog_new ((GtkWindow*) main_window, GTK_FILE_CHOOSER_ACTION_OPEN);
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Browse"));
+	gtk_window_set_title(GTK_WINDOW (dialog), _("Browse"));
 #endif
 	if (user_data != NULL) gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), user_data);
 
@@ -1445,7 +1491,7 @@ void on_export1_activate(GtkMenuItem *menuitem, gpointer user_data) {
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
 #else
 	dialog = hildon_file_chooser_dialog_new ((GtkWindow*) main_window, GTK_FILE_CHOOSER_ACTION_SAVE);
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Browse"));
+	gtk_window_set_title(GTK_WINDOW (dialog), _("Browse"));
 #endif
 	filename = g_strconcat(groups[i], ".grsync", NULL);
 	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), filename);
@@ -1523,6 +1569,6 @@ void on_cellrenderertoggle1_toggled(GtkCellRendererToggle *cell, gchar *path_str
 	gtk_tree_model_get_iter((GtkTreeModel*) liststore_session, &iter, path);
 	gtk_tree_path_free(path);
 	gtk_tree_model_get((GtkTreeModel*) liststore_session, &iter, 1, &enabled, -1);
-	enabled ^= 1;
+	enabled = !enabled;
 	gtk_list_store_set(liststore_session, &iter, 1, enabled, -1);
 }
